/* -*-c++-*-
 * 
 *    Source     $RCSfile: indexmat.h,v $ 
 *    Version    $Revision: 1.1.1.1.2.1 $ 
 *    Date       $Date: 2000/02/28 13:55:17 $ 
 *    Author     Christoph Helmberg 
 * 
 --------------------------------------------------------------------------- */
// ****************************************************************************
//
//  This file is part of the class library "SBmethod", an implementation 
//  of the spectral bundle method for eigenvalue optimization.
//
//  Copyright (C) 2000  Christoph Helmberg
//
//     Konrad-Zuse-Zentrum fuer Informationstechnik Berlin
//     Takustrasse 7, D-14195 Berlin, Germany
//      
//     helmberg@zib.de     http://www.zib.de/helmberg 
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// ****************************************************************************

#ifndef CH_MATRIX_CLASSES__INDEXMAT_HXX
#define CH_MATRIX_CLASSES__INDEXMAT_HXX

#include <iostream>
#include <iomanip>
#include <vector>
#include <math.h>
#ifndef CH_MATRIX_CLASSES__MEMARRAY_HXX
#include "memarray.hxx"
#endif 
#ifndef CH_MATRIX_CLASSES__MYMATH_HXX
#include "mymath.hxx"
#endif


namespace CH_Matrix_Classes {

//everything involving a "Sparsesym" is implemented in "sparssym.cxx/hxx"
//everything else involving a "Sparsemat" is implemented in "sparsmat.cxx/hxx"
//everything else involving a "Symmatrix" is implemented in "symmat.cxx/hxx"
//everything else involving a "Matrix" is implemented in "matrix.cxx/hxx"

// **************************************************************************
//                               Range
// **************************************************************************

//specifies a range of numbers: {i: i=from+k*step for some k\in N_0 and i<=to}

class Range
{
public:
    Integer from;
    Integer to;
    Integer step;
    Range(Integer f,Integer t,Integer s=1):from(f),to(t),step(s){}
    ~Range(){} 
};


// **************************************************************************
//                              Indexmatrix
// **************************************************************************

class Matrix;
class Symmatrix;
class Sparsemat;
class Sparsesym;

class Indexmatrix: protected Memarrayuser
{
    
  friend class Matrix;
  friend class Symmatrix;
  friend class Sparsemat;
  friend class Sparsesym;
    
private:
    Mtype mtype;
    Integer mem_dim;   //amount of memory currently allocated
    Integer nr,nc;     //number rows, number columns
    Integer *m;        //stored columnwise (a11,a21,...,anr1,a12,a22,.....)

    bool is_init;      //flag whether memory is initialized,
                       //it is only used if DEBUG is defined

    inline void init_to_zero();    // (inline)
    
public:

  //----------------------------------------------------
  //----  constructors, destructor, and initialization
  //----------------------------------------------------

  inline Indexmatrix();                              
  inline Indexmatrix(const Indexmatrix&,Integer d=1);       
  inline Indexmatrix(const Range&);              
  inline Indexmatrix(Integer nr,Integer nc);         
  inline Indexmatrix(Integer nr,Integer nc,Integer d);  
  inline Indexmatrix(Integer nr,Integer nc,const Integer* dp,Integer incr=1); 
  inline Indexmatrix(const std::vector<Integer>& vec);
  ~Indexmatrix(){memarray->free(m);}
  
#ifdef DEBUG
  void set_init(bool i){is_init=i;}
  bool get_init() const {return is_init;}
#else 
  void set_init(bool i){}
  bool get_init() const {return true;}
#endif
  
  inline Indexmatrix& init(const Indexmatrix &A,Integer d=1);                   
  Indexmatrix& init(const Range&);
  inline Indexmatrix& init(Integer r,Integer c,Integer d);                    
  inline Indexmatrix& init(Integer r,Integer c,const Integer *dp,Integer incr=1);
  inline Indexmatrix& init(const std::vector<Integer>& vec);
    
  void newsize(Integer r,Integer c);  //resize matrix without initialization


  //----------------------------------------------------
  //----  size and type information
  //----------------------------------------------------

  void dim(Integer& r, Integer& c) const {r=nr; c=nc;}
  Integer dim() const {return nr*nc;}
  Integer rowdim() const {return nr;}
  Integer coldim() const {return nc;}
  
  Mtype get_mtype() const {return mtype;}

    
  //--------------------------------
  //----  Indexing and Submatrices
  //--------------------------------

  inline Integer& operator()(Integer i,Integer j);      
  inline Integer& operator()(Integer i);      //index to vector of stacked columns
  inline Integer operator()(Integer i,Integer j) const; 
  inline Integer operator()(Integer i) const; //index to vector of stacked columns
  Indexmatrix operator()(const Indexmatrix& vecrow,const Indexmatrix& veccol) const;
      //extract submatrix, both indexmatrices are interpreted as vectors
      //repeated elements are allowed
  Indexmatrix operator()(const Indexmatrix& A) const;
      //*this matrix is interpreted as vector, the resulting matrix has
      //has the same shape as A
  
  inline Integer& operator[](Integer i);      //{return (*this)(i);}
  inline Integer operator[](Integer i) const; //{return (*this)(i);}
  
  Indexmatrix col(Integer i) const;          //returns column i as column vector
  Indexmatrix row(Integer i) const;          //returns row i as row vector
  Indexmatrix cols(const Indexmatrix &vec) const;  //returns cols as indexed by vec
  Indexmatrix rows(const Indexmatrix &vec) const;  //returns rows as indexed by vec

  friend Indexmatrix diag(const Indexmatrix& A);      //=(A(1,1),A(2,2),...)^t

  Indexmatrix& triu(Integer d=0);    // (*this)(i,j)=0 for i<j+d
  Indexmatrix& tril(Integer d=0);    // (*this)(i,j)=0 for i>j+d
  friend inline Indexmatrix triu(const Indexmatrix& A,Integer i=0);
  friend inline Indexmatrix tril(const Indexmatrix& A,Integer i=0);
  
  Indexmatrix& subassign(const Indexmatrix& vecrow,const Indexmatrix& veccol,
		    const Indexmatrix& A);
     //(*this)(vecrow(i),veccol(j))=A(i,j) for all i,j
  Indexmatrix& subassign(const Indexmatrix& vec,const Indexmatrix& A);
     //(*this)(vec(i))=A(i);
     //*this, vec, and A may be rect matrices but will be used as vectors
  
  Indexmatrix& delete_rows(const Indexmatrix& ind);
  Indexmatrix& delete_cols(const Indexmatrix& ind);

  Indexmatrix& insert_row(Integer i,const Indexmatrix& v); // 0<=i<=nr, v vector
  Indexmatrix& insert_col(Integer i,const Indexmatrix& v); // 0<=i<=nc, v vector

  inline Indexmatrix& reduce_length(Integer n);  
     //interpret *this as a column vector and cut at n 

  Indexmatrix& concat_right(const Indexmatrix& A);
  Indexmatrix& concat_below(const Indexmatrix& A);
  friend inline Indexmatrix concat_right(const Indexmatrix& A,const Indexmatrix& B);
  friend inline Indexmatrix concat_below(const Indexmatrix& A,const Indexmatrix& B);
  Indexmatrix& concat_below(Integer i);   //only for column vectors (nr==1)
  Indexmatrix& concat_right(Integer i);   //only for row vectors (nc==1)

  friend void swap(Indexmatrix &A,Indexmatrix &B);

  Integer* get_store() {return m;}       //use cautiously, do not use delete!
  const Integer* get_store() const {return m;}   //use cautiously

  
  //------------------------------
  //----  BLAS-like Routines
  //------------------------------

  Indexmatrix& xeya(const Indexmatrix& A,Integer d=1);   //*this=d*A
  Indexmatrix& xpeya(const Indexmatrix& A,Integer d=1);  //*this+=d*A;
    
  friend Indexmatrix& xbpeya(Indexmatrix& x,const Indexmatrix& y,Integer alpha=1,Integer beta=0,int ytrans=0);
  //returns x= alpha*y+beta*x, where y may be transposed (ytrans=1)
  //if beta==0. then x is initialized to the correct size
  
  friend Indexmatrix& xeyapzb(Indexmatrix& x,const Indexmatrix& y,const Indexmatrix& z,Integer alpha=1,Integer beta=1);
  //returns x= alpha*y+beta*z
  //x is initialized to the correct size
  
  friend Indexmatrix& genmult(const Indexmatrix& A,const Indexmatrix& B,Indexmatrix& C,
			 Integer alpha=1,Integer beta=0,int atrans=0,int btrans=0);
  //returns C=beta*C+alpha*A*B, where A and B may be transposed
  //C may neither be equal to A nor B
  //if beta==0 then C is initialized to the correct size
    

  //------------------------------
  //----  usual operators
  //------------------------------

  inline Indexmatrix& operator=(const Indexmatrix &A);
  inline Indexmatrix& operator*=(const Indexmatrix &s);
  inline Indexmatrix& operator+=(const Indexmatrix &v);
  inline Indexmatrix& operator-=(const Indexmatrix &v);
  inline Indexmatrix& operator%=(const Indexmatrix &A);   //Hadamard product!!
  inline Indexmatrix operator-() const;
  
  inline Indexmatrix& operator*=(Integer d);
  inline Indexmatrix& operator/=(Integer d);
  inline Indexmatrix& operator%=(Integer d); //modulo
  inline Indexmatrix& operator+=(Integer d);
  inline Indexmatrix& operator-=(Integer d);
  
  Indexmatrix& transpose();            //transposes itself
  
  friend inline Indexmatrix operator*(const Indexmatrix &A,const Indexmatrix& B);
  friend inline Indexmatrix operator+(const Indexmatrix &A,const Indexmatrix& B);
  friend inline Indexmatrix operator-(const Indexmatrix &A,const Indexmatrix& B);
  friend inline Indexmatrix operator%(const Indexmatrix &A,const Indexmatrix& B);
  friend inline Indexmatrix operator*(const Indexmatrix &A,Integer d);
  friend inline Indexmatrix operator*(Integer d,const Indexmatrix &A);
  friend inline Indexmatrix operator/(const Indexmatrix& A,Integer d);
  friend inline Indexmatrix operator%(const Indexmatrix& A,Integer d);
  friend inline Indexmatrix operator+(const Indexmatrix& A,Integer d);
  friend inline Indexmatrix operator+(Integer d,const Indexmatrix& A);
  friend inline Indexmatrix operator-(const Indexmatrix& A,Integer d);
  friend inline Indexmatrix operator-(Integer d,const Indexmatrix& A);

  friend Indexmatrix transpose(const Indexmatrix& A);

  //------------------------------------------
  //----  Connections to other Matrix Classes
  //------------------------------------------

  friend inline std::vector<int>& assign(std::vector<int>& vec,
					    const Indexmatrix& A); 
  friend inline std::vector<long>& assing(std::vector<long>& vec,
					     const Indexmatrix& A); 

  Indexmatrix(const Matrix&);          //copy with rounding
  Indexmatrix(const Symmatrix&);       //copy with rounding
  Indexmatrix(const Sparsemat&);       //copy with rounding
  Indexmatrix(const Sparsesym&);       //copy with rounding

  //------------------------------
  //----  Elementwise Operations
  //------------------------------

  Indexmatrix& rand(Integer nr,Integer nc,Integer lowerb,Integer upperb);
  Indexmatrix& sign(void);
  Indexmatrix& abs(void);        

  friend inline Indexmatrix rand(Integer nr,Integer nc,Integer lb,Integer ub);
  friend inline Indexmatrix sign(const Indexmatrix& A);
  friend Indexmatrix abs(const Indexmatrix& A);                 

  //----------------------------
  //----  Numerical Methods
  //----------------------------

  friend Integer trace(const Indexmatrix& A);               //=sum(diag(A))
  friend inline Integer ip(const Indexmatrix& A, const Indexmatrix& B); //=trace(B^t*A)
  friend inline Real norm2(const Indexmatrix& A);               //=sqrt(ip(A,A));

  Indexmatrix& scale_rows(const Indexmatrix& vec);    //A=diag(vec)*A
  Indexmatrix& scale_cols(const Indexmatrix& vec);    //A=A*diag(vec)
  
  friend Indexmatrix sumrows(const Indexmatrix& A);   //=(1 1 1 ... 1)*A
  friend Indexmatrix sumcols(const Indexmatrix& A);   //=A*(1 1 ... 1)^t
  friend Integer sum(const Indexmatrix& A);         //=(1 1 ... 1)*A*(1 1 ... 1)^t

  //---------------------------------------------
  //----  Comparisons / Max / Min / sort / find
  //---------------------------------------------

  friend Indexmatrix operator<(const Indexmatrix &A,const Indexmatrix &B);
  friend inline Indexmatrix operator>(const Indexmatrix &A,const Indexmatrix &B);
  friend Indexmatrix operator<=(const Indexmatrix &A,const Indexmatrix &B);
  friend inline Indexmatrix operator>=(const Indexmatrix &A,const Indexmatrix &B);
  friend Indexmatrix operator==(const Indexmatrix &A,const Indexmatrix &B);
  friend Indexmatrix operator!=(const Indexmatrix &A,const Indexmatrix &B);
  friend Indexmatrix operator<(const Indexmatrix &A,Integer d);
  friend Indexmatrix operator>(const Indexmatrix &A,Integer d);
  friend Indexmatrix operator<=(const Indexmatrix &A,Integer d);
  friend Indexmatrix operator>=(const Indexmatrix &A,Integer d);
  friend Indexmatrix operator==(const Indexmatrix &A,Integer d);
  friend Indexmatrix operator!=(const Indexmatrix &A,Integer d);
  friend inline Indexmatrix operator<(Integer d,const Indexmatrix &A);
  friend inline Indexmatrix operator>(Integer d,const Indexmatrix &A);
  friend inline Indexmatrix operator<=(Integer d,const Indexmatrix &A);
  friend inline Indexmatrix operator>=(Integer d,const Indexmatrix &A);
  friend inline Indexmatrix operator==(Integer d,const Indexmatrix &A);
  friend inline Indexmatrix operator!=(Integer d,const Indexmatrix &A);
    
  friend Indexmatrix minrows(const Indexmatrix& A);   //min of each column (over the rows)
  friend Indexmatrix mincols(const Indexmatrix& A);   //min of each row (over the columns)
  friend Integer min(const Indexmatrix& A,Integer *iindex=0,Integer *jindex=0);
  friend Indexmatrix maxrows(const Indexmatrix& A);   //similar
  friend Indexmatrix maxcols(const Indexmatrix& A);   //similar
  friend Integer max(const Indexmatrix& A,Integer *iindex=0,Integer *jindex=0);
  
  friend Indexmatrix sortindex(const Indexmatrix& vec);
  //first element of Indexmatrix points to smallest value of vec, etc.
  friend void sortindex(const Indexmatrix& vec,Indexmatrix &ind);
  //first element of ind points to smallest value of vec, etc.
  //contents of ind on input is irrelevant
  
  Indexmatrix find() const;   //finds nonzeros
  Indexmatrix find_number(Integer num=0) const; 
  friend inline Indexmatrix find(const Indexmatrix& A);
  friend inline Indexmatrix find_number(const Indexmatrix& A,Integer num=0);


  //--------------------------------
  //----  Input / Output
  //--------------------------------

  void display(std::ostream& out,
	       int precision=0,int width=0,int screenwidth=0) const;
  //for variables of value zero default values are used
  //precision=4,width=precision+6,screenwidth=80
  

  friend std::ostream& operator<<(std::ostream& o,const Indexmatrix &v);
  friend std::istream& operator>>(std::istream& i,Indexmatrix &v);
};


// **************************************************************************
//                make non inline friends available outside
// **************************************************************************

Indexmatrix diag(const Indexmatrix& A);      //=(A(1,1),A(2,2),...)^t
void swap(Indexmatrix &A,Indexmatrix &B);
Indexmatrix& xbpeya(Indexmatrix& x,const Indexmatrix& y,Integer alpha,Integer beta,int ytrans);
Indexmatrix& xeyapzb(Indexmatrix& x,const Indexmatrix& y,const Indexmatrix& z,Integer alpha,Integer beta);
Indexmatrix& genmult(const Indexmatrix& A,const Indexmatrix& B,Indexmatrix& C,
			 Integer alpha,Integer beta,int atrans,int btrans);
Indexmatrix transpose(const Indexmatrix& A);
Indexmatrix abs(const Indexmatrix& A);                 
Integer trace(const Indexmatrix& A);               //=sum(diag(A))
Indexmatrix sumrows(const Indexmatrix& A);   //=(1 1 1 ... 1)*A
Indexmatrix sumcols(const Indexmatrix& A);   //=A*(1 1 ... 1)^t
Integer sum(const Indexmatrix& A);         //=(1 1 ... 1)*A*(1 1 ... 1)^t
Indexmatrix operator<(const Indexmatrix &A,const Indexmatrix &B);
Indexmatrix operator<=(const Indexmatrix &A,const Indexmatrix &B);
Indexmatrix operator==(const Indexmatrix &A,const Indexmatrix &B);
Indexmatrix operator!=(const Indexmatrix &A,const Indexmatrix &B);
Indexmatrix operator<(const Indexmatrix &A,Integer d);
Indexmatrix operator>(const Indexmatrix &A,Integer d);
Indexmatrix operator<=(const Indexmatrix &A,Integer d);
Indexmatrix operator>=(const Indexmatrix &A,Integer d);
Indexmatrix operator==(const Indexmatrix &A,Integer d);
Indexmatrix operator!=(const Indexmatrix &A,Integer d);
Indexmatrix minrows(const Indexmatrix& A);   //min of each column (over the rows)
Indexmatrix mincols(const Indexmatrix& A);   //min of each row (over the columns)
Integer min(const Indexmatrix& A,Integer *iindex,Integer *jindex);
Indexmatrix maxrows(const Indexmatrix& A);   //similar
Indexmatrix maxcols(const Indexmatrix& A);   //similar
Integer max(const Indexmatrix& A,Integer *iindex,Integer *jindex);
Indexmatrix sortindex(const Indexmatrix& vec);
void sortindex(const Indexmatrix& vec,Indexmatrix &ind);
std::ostream& operator<<(std::ostream& o,const Indexmatrix &v);
std::istream& operator>>(std::istream& i,Indexmatrix &v);




// **************************************************************************
//                   implementation of inline functions
// **************************************************************************

inline void Indexmatrix::init_to_zero()
{
 mtype=MTindexmatrix;
 nr=nc=0;mem_dim=0;m=0;
 chk_set_init(*this,1);
}

inline Indexmatrix& Indexmatrix::init(Integer inr,Integer inc,Integer d)
{
 newsize(inr,inc);
 mat_xea(nr*nc,m,d);
 chk_set_init(*this,1);
 return *this;
}

inline Indexmatrix& Indexmatrix::init(Integer inr,Integer inc,const Integer* d,Integer incr)
{
 newsize(inr,inc);
 if (incr==1) mat_xey(nr*nc,m,d);
 else mat_xey(nr*nc,m,Integer(1),d,incr);
 chk_set_init(*this,1);
 return *this;
}

inline Indexmatrix& Indexmatrix::init(const Indexmatrix& A,Integer d)
{
 return xeya(A,d);
}

inline Indexmatrix& Indexmatrix::init(const std::vector<Integer>& vec)
{
  newsize(vec.size(),1); chk_set_init(*this,1);
  for(Integer i=0;i<nr;i++) m[i]=vec[i];
  return *this;
}

inline Indexmatrix::Indexmatrix()
{
 init_to_zero();
}

inline Indexmatrix::Indexmatrix(const Indexmatrix &A,Integer d)
{
 init_to_zero();
 xeya(A,d);
}

inline Indexmatrix::Indexmatrix(Integer inr,Integer inc)
{
 init_to_zero();
 newsize(inr,inc);
}

inline Indexmatrix::Indexmatrix(const Range& range)
{
 init_to_zero();
 init(range);
}

inline Indexmatrix::Indexmatrix(Integer inr,Integer inc,Integer d)
{
 init_to_zero();
 init(inr,inc,d);
}

inline Indexmatrix::Indexmatrix(Integer inr,Integer inc,const Integer *d,Integer incr)
{
 init_to_zero();
 init(inr,inc,d,incr);
}

inline Indexmatrix::Indexmatrix(const std::vector<Integer>& vec)
{
 init_to_zero();
 init(vec);
}

inline Integer& Indexmatrix::operator()(Integer i,Integer j)
{
 chk_range(i,j,nr,nc);
 return m[j*nr+i];
}

inline Integer& Indexmatrix::operator()(Integer i) 
{
 chk_range(i,0,nr*nc,1);
 return m[i];
}

inline Integer Indexmatrix::operator()(Integer i,Integer j) const
{
 chk_range(i,j,nr,nc);
 return m[j*nr+i];
}

inline Integer Indexmatrix::operator()(Integer i) const
{
 chk_range(i,0,nr*nc,1);
 return m[i];
}

inline Integer& Indexmatrix::operator[](Integer i)
{return (*this)(i);}

inline Integer Indexmatrix::operator[](Integer i) const 
{return (*this)(i);}

inline Indexmatrix& Indexmatrix::reduce_length(Integer n) 
{ nr=min(nr*nc,max(Integer(0),n)); nc=1; return *this;}


inline Integer ip(const Indexmatrix& A, const Indexmatrix& B)
{
 chk_add(A,B);
 return mat_ip(A.nc*A.nr,A.m,B.m);
}

inline Real norm2(const Indexmatrix& A)
{
 chk_init(A);
 return ::sqrt(double(mat_ip(A.nc*A.nr,A.m,A.m)));
}

inline Indexmatrix& Indexmatrix::operator=(const Indexmatrix &A)
{ return xeya(A);}

inline Indexmatrix& Indexmatrix::operator*=(const Indexmatrix &A)
{ Indexmatrix C; return xeya(genmult(*this,A,C));}

inline Indexmatrix& Indexmatrix::operator+=(const Indexmatrix &A)
{ return xpeya(A); }

inline Indexmatrix& Indexmatrix::operator-=(const Indexmatrix &A)
{ return xpeya(A,-1); }

inline Indexmatrix& Indexmatrix::operator%=(const Indexmatrix &A)
{ chk_add(*this,A); mat_xhadey(nr*nc,m,A.m); return *this; }

inline Indexmatrix Indexmatrix::operator-() const
{ return Indexmatrix(*this,-1); }

inline Indexmatrix& Indexmatrix::operator*=(register Integer d)
{ chk_init(*this); mat_xmultea(nr*nc,m,d); return *this; }

inline Indexmatrix& Indexmatrix::operator/=(register Integer d)
{ chk_init(*this); mat_xdivea(nr*nc,m,d); return *this; }

inline Indexmatrix& Indexmatrix::operator%=(register Integer d)
{ chk_init(*this); mat_xmodea(nr*nc,m,d); return *this; }

inline Indexmatrix& Indexmatrix::operator+=(register Integer d)
{ chk_init(*this); mat_xpea(nr*nc,m,d); return *this; }

inline Indexmatrix& Indexmatrix::operator-=(register Integer d)
{ chk_init(*this); mat_xpea(nr*nc,m,-d); return *this; }


inline Indexmatrix operator*(const Indexmatrix &A,const Indexmatrix &B) 
    {Indexmatrix C; return genmult(A,B,C);}
inline Indexmatrix operator+(const Indexmatrix &A,const Indexmatrix &B)
    {Indexmatrix C; return xeyapzb(C,A,B,1,1);}
inline Indexmatrix operator-(const Indexmatrix &A,const Indexmatrix &B)
    {Indexmatrix C; return xeyapzb(C,A,B,1,-1);}
inline Indexmatrix operator%(const Indexmatrix &A,const Indexmatrix &B) 
    {Indexmatrix C(A); return C%=B;}
inline Indexmatrix operator*(const Indexmatrix &A,Integer d)          
    {return Indexmatrix(A,d);}
inline Indexmatrix operator*(Integer d,const Indexmatrix &A)
    {return Indexmatrix(A,d);}
inline Indexmatrix operator/(const Indexmatrix &A,Integer d)
    {Indexmatrix B(A); return B/=d;}
inline Indexmatrix operator%(const Indexmatrix &A,Integer d)
    {Indexmatrix B(A); return B%=d;}
inline Indexmatrix operator+(const Indexmatrix &A,Integer d)           
    {Indexmatrix B(A); return B+=d;}
inline Indexmatrix operator+(Integer d,const Indexmatrix &A)           
    {Indexmatrix B(A); return B+=d;}
inline Indexmatrix operator-(const Indexmatrix &A,Integer d)       
    {Indexmatrix B(A); return B-=d;}
inline Indexmatrix operator-(Integer d,const Indexmatrix &A)       
    {Indexmatrix B(A,-1); return B+=d;}

inline Indexmatrix triu(const Indexmatrix& A,Integer i)
          {Indexmatrix B(A); B.triu(i); return B;}
inline Indexmatrix tril(const Indexmatrix& A,Integer i)
          {Indexmatrix B(A); B.tril(i); return B;}

inline Indexmatrix concat_right(const Indexmatrix& A,const Indexmatrix& B)
          {Indexmatrix C(A.dim()+B.dim(),1);C=A;C.concat_right(B);return C;}
inline Indexmatrix concat_below(const Indexmatrix& A,const Indexmatrix& B)
          {Indexmatrix C(A.dim()+B.dim(),1);C=A;C.concat_below(B);return C;}
    
inline void swap(Indexmatrix &A,Indexmatrix &B)
{ 
 Integer *hm=A.m;A.m=B.m;B.m=hm;
 Integer hi=A.nr;A.nr=B.nr;B.nr=hi;
 hi=A.nc;A.nc=B.nc;B.nc=hi;
 hi=A.mem_dim;A.mem_dim=B.mem_dim;B.mem_dim=hi;
#ifdef DEBUG
 hi=A.is_init;A.is_init=B.is_init;B.is_init=hi;
#endif
}

inline Indexmatrix rand(Integer rows,Integer cols,Integer lb,Integer ub)
          {Indexmatrix A; return A.rand(rows,cols,lb,ub);}
inline Indexmatrix sign(const Indexmatrix& A)
          {Indexmatrix B(A); return B.sign();}

inline Indexmatrix operator>(const Indexmatrix &A,const Indexmatrix &B)
{return B<A;}
inline Indexmatrix operator>=(const Indexmatrix &A,const Indexmatrix &B)
{return B<=A;}
inline Indexmatrix operator<(Integer d,const Indexmatrix &A)
{return A>d;}
inline Indexmatrix operator>(Integer d,const Indexmatrix &A)
{return A<d;}
inline Indexmatrix operator<=(Integer d,const Indexmatrix &A)
{return A>=d;}
inline Indexmatrix operator>=(Integer d,const Indexmatrix &A)
{return A<=d;}
inline Indexmatrix operator==(Integer d,const Indexmatrix &A)
{return A==d;}
inline Indexmatrix operator!=(Integer d,const Indexmatrix &A)
{return A!=d;}

inline Indexmatrix find(const Indexmatrix& A)
          {return A.find();}
inline Indexmatrix find_number(const Indexmatrix& A,Integer num)
          {return A.find_number(num);}

inline std::vector<int>& assign(std::vector<int>& vec,const Indexmatrix& A)
{
  chk_init(A);
  vec.resize(A.dim());
  for(Integer i=0;i<A.dim();i++) vec[i]=int(A(i));
  return vec;
}

inline std::vector<long>& assign(std::vector<long>& vec,const Indexmatrix& A)
{
  chk_init(A);
  vec.resize(A.dim());
  for(Integer i=0;i<A.dim();i++) vec[i]=long(A(i));
  return vec;
}

}

#ifndef CH_MATRIX_CLASSES__MATRIX_HXX
#include "matrix.hxx"
#endif


#endif

/* ---------------------------------------------------------------------------
 *    Change log $Log: indexmat.h,v $
 *    Change log Revision 1.1.1.1.2.1  2000/02/28 13:55:17  bzfhelmb
 *    Change log Reording inline functions and correction of minor details
 *    Change log
 *    Change log Revision 1.1.1.1  1999/12/15 13:47:53  bzfhelmb
 *    Change log Imported sources
 *    Change log
 *
 *    End of $Source: /kombadon/cvs/cvsroot/bzfhelmb/SBmethod/matrix/indexmat.h,v $
 --------------------------------------------------------------------------- */
