/* -*-c++-*-
 * 
 *    Source     $RCSfile: matop.h,v $ 
 *    Version    $Revision: 1.1.1.1 $ 
 *    Date       $Date: 1999/12/15 13:47:54 $ 
 *    Author     Christoph Helmberg 
 * 
 --------------------------------------------------------------------------- */

// ****************************************************************************
//
//  This file is part of the class library "SBmethod", an implementation 
//  of the spectral bundle method for eigenvalue optimization.
//
//  Copyright (C) 2000  Christoph Helmberg
//
//     Konrad-Zuse-Zentrum fuer Informationstechnik Berlin
//     Takustrasse 7, D-14195 Berlin, Germany
//      
//     helmberg@zib.de     http://www.zib.de/helmberg 
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// ****************************************************************************

#ifndef CH_MATRIX_CLASSES__MATOP_HXX
#define CH_MATRIX_CLASSES__MATOP_HXX

#include <limits.h>
#include <float.h>
#include <assert.h>
#ifdef WITH_BLAS
extern "C"{
#include "cblas.hxx"
}
#endif
#include "gb_rand.hxx"


namespace CH_Matrix_Classes {

typedef int Integer;
const Integer max_Integer= INT_MAX;
const Integer min_Integer= INT_MIN;

typedef double Real;
const Real max_Real= DBL_MAX;
const Real min_Real= -DBL_MAX;
const Real eps_Real= DBL_EPSILON;


#ifdef WITH_BLAS
 inline void mat_xey( Integer len, Real *x, const Real* y)
 {
   cblas_dcopy(len,y,1,x,1);
 }
#endif

template<class Val>
inline void mat_xey( Integer len, Val *x, const Val* y)
{
 for(;--len>=0;) (*x++)=(*y++);
}

#ifdef WITH_BLAS
 inline void mat_xey( Integer len, Real *x, const Integer incx, 
		      const Real* y, const Integer incy)
 {
   cblas_dcopy(len,y,incy,x,incx);
 }
#endif

template<class Val>
inline void mat_xey( Integer len, Val *x, const Integer incx,
                     const Val *y, const Integer incy)
{
 for(;--len>=0;x+=incx,y+=incy) (*x)=(*y);
}

#ifdef WITH_BLAS
inline void mat_xemx( Integer len, double *x)
 {
   cblas_dscal(len,-1.,x,1);
 }
#endif

template<class Val>
inline void mat_xemx( Integer len, Val *x)
{
 for(;--len>=0;x++) (*x)=-(*x);
}

#ifdef WITH_BLAS
inline void mat_xemx( Integer len, double *x, const Integer incx)
 {
   cblas_dscal(len,-1.,x,incx);
 }
#endif

template<class Val>
inline void mat_xemx( Integer len, Val *x, const Integer incx)
{
 for(;--len>=0;x+=incx) (*x)=-(*x);
}

#ifdef WITH_BLAS
inline void mat_xemy( Integer len, double *x, const double *y)
 {
   cblas_dcopy(len,y,1,x,1);
   cblas_dscal(len,-1.,x,1);
 }
#endif

template<class Val>
inline void mat_xemy( Integer len, Val *x, const Val *y)
{
 for(;--len>=0;) (*x++)=-(*y++);
}

#ifdef WITH_BLAS
inline void mat_xemy( Integer len, double *x, const Integer incx,
		      const double *y, const Integer incy)
 {
   cblas_dcopy(len,y,incy,x,incx);
   cblas_dscal(len,-1.,x,incx);
 }
#endif

template<class Val>
inline void mat_xemy( Integer len, Val *x, const Integer incx,
                          const Val *y, const Integer incy)
{
 for(;--len>=0;x+=incx,y+=incy) (*x)=-(*y);
}

#ifdef WITH_BLAS
inline void mat_xeya( Integer len, double *x,
                     const double *y, double d)
 {
   cblas_dcopy(len,y,1,x,1);
   cblas_dscal(len,d,x,1);
 }
#endif

template<class Val>
inline void mat_xeya( Integer len, Val *x,
                     const Val *y, Val d)
{
 for(;--len>=0;) (*x++)=d*(*y++);
}

#ifdef WITH_BLAS
inline void mat_xeya( Integer len, double *x, const Integer incx,
		      const double *y, const Integer incy,double d)
 {
   cblas_dcopy(len,y,incy,x,incx);
   cblas_dscal(len,d,x,incx);
 }
#endif

template<class Val>
inline void mat_xeya( Integer len, Val *x, const Integer incx,
                     const Val *y, const Integer incy, Val d)
{
 for(;--len>=0;x+=incx,y+=incy) (*x)=d*(*y);
}

#ifdef WITH_BLAS
inline void mat_xpey( Integer len, double *x,
		      const double *y)
 {
   cblas_daxpy(len,1.,y,1,x,1);
 }
#endif

template<class Val>
inline void mat_xpey( Integer len, Val *x, const Val *y)
{
 for(;--len>=0;) (*x++)+=(*y++);
}

#ifdef WITH_BLAS
inline void mat_xpey( Integer len, double *x, const Integer incx,
		      const double *y, const Integer incy)
 {
   cblas_daxpy(len,1.,y,incy,x,incx);
 }
#endif

template<class Val>
inline void mat_xpey( Integer len, Val *x, const Integer incx,
                         const Val *y, const Integer incy)
{
 for(;--len>=0;x+=incx,y+=incy) (*x)+=(*y);
}

#ifdef WITH_BLAS
inline void mat_xmey( Integer len, double *x,
		      const double *y)
 {
   cblas_daxpy(len,-1.,y,1,x,1);
 }
#endif

template<class Val>
inline void mat_xmey( Integer len, Val *x, const Val *y)
{
 for(;--len>=0;) (*x++)-=(*y++);
}

#ifdef WITH_BLAS
inline void mat_xmey( Integer len, double *x, const Integer incx,
		      const double *y, const Integer incy)
 {
   cblas_daxpy(len,-1.,y,incy,x,incx);
 }
#endif

template<class Val>
inline void mat_xmey( Integer len, Val *x, const Integer incx,
                          const Val *y, const Integer incy)
{
 for(;--len>=0;x+=incx,y+=incy) (*x)-=(*y);
}

template<class Val>
inline void mat_xhadey( Integer len, Val *x, const Val *y)
{
 for(;--len>=0;) (*x++)*=(*y++);
}

template<class Val>
inline void mat_xhadey( Integer len, Val *x, const Integer incx,
                          const Val *y, const Integer incy)
{
 for(;--len>=0;x+=incx,y+=incy) (*x)*=(*y);
}

#ifdef WITH_BLAS
inline void mat_xpeya( Integer len, Real *x,
		       const Real *y, Real d)
{
   cblas_daxpy(len,d,y,1,x,1);
 }
#endif

template<class Val>
inline void mat_xpeya( Integer len, Val *x,
                         const Val *y, Val d)
{
 for(;--len>=0;) (*x++)+=d*(*y++);
}

#ifdef WITH_BLAS
inline void mat_xpeya( Integer len, Real *x, const Integer incx,
                         const Real *y, const Integer incy, Real d)
{
  cblas_daxpy(len,d,y,incy,x,incx);
 }
#endif

template<class Val>
inline void mat_xpeya( Integer len, Val *x, const Integer incx,
                         const Val *y, const Integer incy, Val d)
{
 for(;--len>=0;x+=incx,y+=incy) (*x)+=d*(*y);
}

#ifdef WITH_BLAS
inline void mat_xbpeya( Integer len, Real *x,
                         const Real *y, Real a, Real b)
{
  cblas_dscal(len,b,x,1);
  cblas_daxpy(len,a,y,1,x,1);
}
#endif

template<class Val>
inline void mat_xbpeya( Integer len, Val *x,
                         const Val *y, Val a,Val b)
{
 for(;--len>=0;x++) (*x)=b*(*x)+a*(*y++);
}

#ifdef WITH_BLAS
inline void mat_xbpeya( Integer len, Real *x, const Integer incx,
                         const Real *y, const Integer incy, Real a, Real b)
{
  cblas_dscal(len,b,x,incx);
  cblas_daxpy(len,a,y,incy,x,incx);
}
#endif

template<class Val>
inline void mat_xbpeya( Integer len, Val *x, const Integer incx,
                         const Val *y, const Integer incy, Val a,Val b)
{
 for(;--len>=0;x+=incx,y+=incy) (*x)+=b*(*x)+a*(*y);
}

template<class Val>
inline void mat_xea( Integer len, Val *x, Val d)
{
 for(;--len>=0;) (*x++)=d;
}

template<class Val>
inline void mat_xea( Integer len, Val *x, const Integer incx,
                     Val d)
{
 for(;--len>=0;x+=incx) (*x)=d;
}

template<class Val>
inline void mat_xpea( Integer len, Val *x, Val d)
{
 for(;--len>=0;) (*x++)+=d;
}

template<class Val>
inline void mat_xpea( Integer len, Val *x, const Integer incx,
                         Val d)
{
 for(;--len>=0;x+=incx) (*x)+=d;
}

#ifdef WITH_BLAS
inline void mat_xmultea( Integer len, double *x,double d)
 {
   cblas_dscal(len,d,x,1);
 }
#endif

template<class Val>
inline void mat_xmultea( Integer len, Val *x, Val d)
{
 for(;--len>=0;) (*x++)*=d;
}

#ifdef WITH_BLAS
inline void mat_xmultea( Integer len, double *x, const Integer incx,double d)
 {
   cblas_dscal(len,d,x,incx);
 }
#endif

template<class Val>
inline void mat_xmultea( Integer len, Val *x, const Integer incx,
                         Val d)
{
 for(;--len>=0;x+=incx) (*x)*=d;
}

#ifdef WITH_BLAS
inline void mat_xdivea( Integer len, double *x,double d)
 {
   cblas_dscal(len,1./d,x,1);
 }
#endif

template<class Val>
inline void mat_xdivea( Integer len, Val *x, Val d)
{
 for(;--len>=0;) (*x++)/=d;
}

#ifdef WITH_BLAS
inline void mat_xdivea( Integer len, double *x, const Integer incx,double d)
 {
   cblas_dscal(len,1./d,x,incx);
 }
#endif

template<class Val>
inline void mat_xdivea( Integer len, Val *x, const Integer incx,
                         Val d)
{
 for(;--len>=0;x+=incx) (*x)/=d;
}

template<class Val>
inline void mat_xmodea( Integer len, Val *x, Val d)
{
 for(;--len>=0;) (*x++)%=d;
}

template<class Val>
inline void mat_xmodea( Integer len, Val *x, const Integer incx,
                         Val d)
{
 for(;--len>=0;x+=incx) (*x)%=d;
}

#ifdef WITH_BLAS
inline void mat_xeypz( Integer len, Real *x,
                         const Real *y,
                         const Real *z, Real a, Real b)
{
  cblas_dcopy(len,y,1,x,1);
  cblas_daxpy(len,1.,z,1,x,1);
}
#endif

template<class Val>
inline void mat_xeypz( Integer len, Val *x,
                      const Val *y, const Val *z)
{
 for(;--len>=0;) (*x++)=(*y++)+(*z++);
}

#ifdef WITH_BLAS
inline void mat_xeypz( Integer len, Real *x, const Integer incx,
                         const Real *y, const Integer incy,
                         const Real *z, const Integer incz)
{
  cblas_dcopy(len,y,incy,x,incx);
  cblas_daxpy(len,1.,z,incz,x,incx);
}
#endif

template<class Val>
inline void mat_xeypz( Integer len, Val *x, const Integer incx,
                       const Val *y, const Integer incy,
                       const Val *z, const Integer incz)
{
 for(;--len>=0;x+=incx,y+=incy,z+=incz) (*x)=(*y)+(*z);
}

#ifdef WITH_BLAS
inline void mat_xeymz( Integer len, Real *x,
                         const Real *y,
                         const Real *z, Real a, Real b)
{
  cblas_dcopy(len,y,1,x,1);
  cblas_daxpy(len,-1.,z,1,x,1);
}
#endif

template<class Val>
inline void mat_xeymz( Integer len, Val *x,
                       const Val *y, const Val *z)
{
 for(;--len>=0;) (*x++)=(*y++)-(*z++);
}

#ifdef WITH_BLAS
inline void mat_xeymz( Integer len, Real *x, const Integer incx,
                         const Real *y, const Integer incy,
                         const Real *z, const Integer incz)
{
  cblas_dcopy(len,y,incy,x,incx);
  cblas_daxpy(len,-1.,z,incz,x,incx);
}
#endif

template<class Val>
inline void mat_xeymz( Integer len, Val *x, const Integer incx,
                       const Val *y, const Integer incy,
                       const Val *z, const Integer incz)
{
 for(;--len>=0;x+=incx,y+=incy,z+=incz) (*x)=(*y)-(*z);
}

#ifdef WITH_BLAS
inline void mat_xeyapzb( Integer len, Real *x,
                         const Real *y,
                         const Real *z, Real a, Real b)
{
  cblas_dcopy(len,y,1,x,1);
  cblas_dscal(len,a,x,1);
  cblas_daxpy(len,b,z,1,x,1);
}
#endif

template<class Val>
inline void mat_xeyapzb( Integer len, Val *x,
                      const Val *y, const Val *z,Val a,Val b)
{
 for(;--len>=0;) (*x++)=a*(*y++)+b*(*z++);
}

#ifdef WITH_BLAS
inline void mat_xeyapzb( Integer len, Real *x, const Integer incx,
                         const Real *y, const Integer incy,
                         const Real *z, const Integer incz, Real a, Real b)
{
  cblas_dcopy(len,y,incy,x,incx);
  cblas_dscal(len,a,x,incx);
  cblas_daxpy(len,b,z,incz,x,incx);
}
#endif

template<class Val>
inline void mat_xeyapzb( Integer len, Val *x, const Integer incx,
                       const Val *y, const Integer incy,
                       const Val *z, const Integer incz,Val a,Val b)
{
 for(;--len>=0;x+=incx,y+=incy,z+=incz) (*x)=a*(*y)+b*(*z);
}

#ifdef WITH_BLAS
inline double mat_ip( Integer len, const double *x,const double *y)
{
  return cblas_ddot(len,x,1,y,1);
}
#endif

template<class Val>
inline Val mat_ip( Integer len, const Val *x, const Val *y)
{
  Val sum=0;
 for(;--len>=0;) sum+=(*x++)*(*y++);
 return sum;
}


#ifdef WITH_BLAS
inline double mat_ip( Integer len, const double *x, const Integer incx,
                   const double *y, const Integer incy)
{
  return cblas_ddot(len,x,incx,y,incy);
}
#endif

template<class Val>
inline Val mat_ip( Integer len, const Val *x, const Integer incx,
                   const Val *y, const Integer incy)
{
  Val sum=0;
 for(;--len>=0;x+=incx,y+=incy) sum+=(*x)*(*y);
 return sum;
}

template<class Val>
inline Val mat_sum( Integer len, const Val *x)
{
  Val sum=0;
 for(;--len>=0;) sum+=(*x++);
 return sum;
}

template<class Val>
inline Val mat_sum( Integer len, const Val *x,const Integer incx)
{
  Val sum=0;
 for(;--len>=0;x+=incx) sum+=(*x);
 return sum;
}

#ifdef WITH_BLAS
inline void mat_swap( Integer len, double *x, double *y)
{
  cblas_dswap(len,x,1,y,1);
}
#endif

template<class Val>
inline void mat_swap( Integer len, Val *x, Val *y)
{
 for(;--len>=0;) {Val h=*x;(*x++)=*y;(*y++)=h;}
}

#ifdef WITH_BLAS
inline void mat_swap( Integer len, double *x, const Integer incx, double *y, const Integer incy)
{
  cblas_dswap(len,x,incx,y,incy);
}
#endif

template<class Val>
inline void mat_swap( Integer len, Val *x, const Integer incx,
                      Val *y, const Integer incy)
{
 for(;--len>=0;x+=incx,y+=incy) {Val h=*x;(*x)=*y;(*y)=h;}
}


//Implementations and Instantiations of the following globals are all
//in memarray.cxx

// **************************************************************************
//                                mat_randgen
// **************************************************************************

//common random number generator used by all matrix-type implementations

extern CH_Tools::GB_rand mat_randgen;
  

// **************************************************************************
//                               Matrix Types
// **************************************************************************

enum Mtype {
    MTglobalfun,
    MTindexmatrix,
    MTmatrix,        
    MTsymmetric,
    MTsparse,
    MTsparsesym
};

// **************************************************************************
//                                materrout
// **************************************************************************

extern std::ostream* materrout; //default: pointer to std::cout


// **************************************************************************
//                               "exceptions"
// **************************************************************************

//the following are not really exceptions but only ment to facilitate
//error detection.
//In a debugger catch the Matrix Errors by setting a breakpoint in MEmessage

enum MEcode {ME_unspec,ME_range,ME_mem,ME_dim,ME_num,ME_warning};

class MatrixError
{
public:
    MEcode code;      
    const char *message;     //must point to existing object, does not get freed.
    Mtype mtype;
    MatrixError(MEcode c=ME_unspec,const char *mes=0,Mtype mt=MTmatrix):
        code(c),message(mes),mtype(mt){}
};

//--- some index is out of range
class MErange:public MatrixError
{
public:
    Integer nr,r;   
    Integer nc,c;
    MErange(Integer inr,Integer ir,Integer inc,Integer ic,const char *mes=0,Mtype mt=MTmatrix):
        MatrixError(ME_range,mes,mt),nr(inr),r(ir),nc(inc),c(ic){}
};

//--- cannot allocate memory
class MEmem:public MatrixError
{
public:
    Integer size;
    MEmem(Integer s,const char *mes=0,Mtype mt=MTmatrix):
        MatrixError(ME_mem,mes,mt),size(s){}
};

//--- matrix dimensions do not agree for desired operation
class MEdim:public MatrixError
{
public:
    Integer nr1, nc1, nr2, nc2;
    MEdim(Integer r1,Integer c1,Integer r2,Integer c2,const char *mes=0,Mtype mt=MTmatrix):
        MatrixError(ME_dim,mes,mt),nr1(r1),nc1(c1),nr2(r2),nc2(c2){}
};

int MEmessage(const MatrixError&); //displays error message and terminates
                                   //or returns 1 in case of warnings

// **************************************************************************
//                         DEBUG error checking templates
// **************************************************************************

#ifdef DEBUG

template<class Mat>
inline void chk_init(const Mat& A)
{
 if (!A.get_init())
     MEmessage(MatrixError(ME_unspec,"Matrix not initialized",A.get_mtype()));
}

template<class Mat1,class Mat2>
inline void chk_add(const Mat1& A,const Mat2& B)
{
 chk_init(A); chk_init(B);
 if ((A.rowdim()!=B.rowdim())||(A.coldim()!=B.coldim()))
     MEmessage(MEdim(A.rowdim(),A.coldim(),B.rowdim(),B.coldim(),
                     "dimensions do not match in additive expression",
                     A.get_mtype()));
}

template<class Mat1,class Mat2>
inline void chk_mult(const Mat1& A,const Mat2& B)
{
 chk_init(A); chk_init(B);
 if (A.coldim()!=B.rowdim())
     MEmessage(MEdim(A.rowdim(),A.coldim(),B.rowdim(),B.coldim(),
                     "dimensions do not match in multiplicative expression",
                     A.get_mtype()));
}

template<class Mat1,class Mat2>
inline void chk_rowseq(const Mat1& A,const Mat2& B)
{
 chk_init(A); chk_init(B);
 if (A.rowdim()!=B.rowdim())
     MEmessage(MEdim(A.rowdim(),A.coldim(),B.rowdim(),B.coldim(),
                     "number of rows do not agree",
                     A.get_mtype()));
}

template<class Mat1,class Mat2>
inline void chk_colseq(const Mat1& A,const Mat2& B)
{
 chk_init(A); chk_init(B);
 if (A.coldim()!=B.coldim())
     MEmessage(MEdim(A.rowdim(),A.coldim(),B.rowdim(),B.coldim(),
                     "number of columns do not agree",
                     A.get_mtype()));
}

inline void chk_range(Integer r,Integer c,Integer ubr,Integer ubc)
{
 if ((r<0)||(c<0)||
     ((ubr>=0)&&(r>=ubr))||
     ((ubc>=0)&&(c>=ubc)))
     MEmessage(MErange(r,ubr,c,ubc,"index out of range or negative dimension"));
}

template<class Mat>
inline void chk_set_init(Mat &A,int val){A.set_init(val);}

#else

// if DEBUG is not definied all chk_functions are mapped to the null string

#define chk_init(x)
#define chk_add(x,y)
#define chk_mult(x,y)
#define chk_rowseq(x,y)
#define chk_colseq(x,y)
#define chk_range(u,v,w,x)
#define chk_set_init(x,y)

#endif 
 



}

#endif

/* ---------------------------------------------------------------------------
 *    Change log $Log: matop.h,v $
 *    Change log Revision 1.1.1.1  1999/12/15 13:47:54  bzfhelmb
 *    Change log Imported sources
 *    Change log
 *
 *    End of $Source: /kombadon/cvs/cvsroot/bzfhelmb/SBmethod/matrix/matop.h,v $
 --------------------------------------------------------------------------- */
